@@ -65,11 +65,21 @@ impl Block {
         context.blocks[self.0].label.clone()
     }
 
+    /// Get the number of instructions in this block, NOT including the phi instruction.
+    pub fn num_instructions(&self, context: &Context) -> usize {
+        context.blocks[self.0].instructions.len() - 1
+    }
+
     /// Get the phi instruction for this block.
     pub fn get_phi(&self, context: &Context) -> Value {
         context.blocks[self.0].instructions[0]
     }
 
+    /// Get the number of predecessor blocks, i.e., blocks which branch to this one.
+    pub fn num_predecessors(&self, context: &Context) -> usize {
+        context.blocks[self.0].num_predecessors(context)
+    }
+
     /// Add a new phi entry to this block.
     ///
     /// This indicates that if control flow comes from `from_block` then the phi instruction should
@@ -246,6 +256,16 @@ impl Block {
     }
 }
 
+#[doc(hidden)]
+impl BlockContent {
+    pub(super) fn num_predecessors(&self, context: &Context) -> usize {
+        match &context.values[self.instructions[0].0].value {
+            ValueDatum::Instruction(Instruction::Phi(list)) => list.len(),
+            _ => unreachable!("First value in block instructions is not a phi."),
+        }
+    }
+}
+
 /// An iterator over each block in a [`Function`].
 pub struct BlockIterator {
     blocks: Vec<generational_arena::Index>,
